@@ -30,8 +30,9 @@ from linaro_dashboard_bundle.io import DocumentIO
 
 from lava_android_test.adb import ADB
 from lava_android_test.config import get_config
+from lava_android_test.provider import TestProvider
 from lava_android_test.repository import GitRepository
-from lava_android_test.testdef import testloader, AndroidTest
+from lava_android_test.testdef import AndroidTest
 from lava_android_test.testdef import AndroidTestRunner, \
                                       AndroidTestInstaller, \
                                       AndroidTestParser
@@ -103,11 +104,10 @@ class list_tests(Command):
     """
 
     def invoke(self):
-        from lava_android_test import test_definitions
-        from pkgutil import walk_packages
         self.say("Known tests:")
-        for importer, mod, ispkg in walk_packages(test_definitions.__path__):
-            self.say(" - {test_id}", test_id=mod)
+        for provider in TestProvider().get_test_provider_list():
+            for test in provider().list_test():
+                self.say(" - {test_id}", test_id=test)
 
 
 class version(Command):
@@ -152,13 +152,7 @@ class AndroidCommand(Command):
         return self.adb.exists(test_dir)
 
     def get_device_serial(self):
-        if not self.args.serial:
-            serial_ary = ADB().run_cmd_host('adb get-serialno')[1]
-            serial = serial_ary[0].strip()
-            if not serial or serial == 'unknown':
-                return ''
-            else:
-                return serial
+        return ADB(self.args.serial).get_serial()
 
     def assertDeviceIsConnected(self):
         if not self.adb.isDeviceConnected():
@@ -169,7 +163,12 @@ class AndroidCommand(Command):
                 raise Exception("No device found")
 
     def invoke(self):
-        self.adb = ADB(self.args.serial)
+        serial = self.get_device_serial()
+        if not serial:
+            raise LavaCommandError("No device attached")
+        self.serial = serial
+        self.adb = ADB(self.serial)
+
         try:
             self.assertDeviceIsConnected()
         except Exception as err:
@@ -272,7 +271,7 @@ class install(AndroidTestCommand):
         if self.test_installed(self.args.test_id):
             raise LavaCommandError("The test (%s) has already installed." %
                                    self.args.test_id)
-        test = testloader(self.args.test_id, self.args.serial)
+        test = TestProvider().load_test(self.args.test_id, self.args.serial)
         try:
             test.install(self.args.install_option)
         except Exception as strerror:
@@ -291,7 +290,7 @@ class uninstall(AndroidTestCommand):
         tip_msg = self.get_tip_msg("Uninstall test")
         self.say_begin(tip_msg)
 
-        test = testloader(self.args.test_id, self.args.serial)
+        test = TestProvider().load_test(self.args.test_id, self.args.serial)
         try:
             test.uninstall()
         except Exception as strerror:
@@ -331,7 +330,7 @@ class run(AndroidTestCommand):
             raise LavaCommandError(
                 "The test (%s) has not been installed yet." %
                 self.args.test_id)
-        test = testloader(self.args.test_id, self.args.serial)
+        test = TestProvider().load_test(self.args.test_id, self.args.serial)
 
         if not self.test_installed(test.testname):
             raise LavaCommandError(
@@ -388,7 +387,6 @@ class run_custom(AndroidCommand):
 
     def invoke_sub(self):
 
-        config = get_config()
         test_name = 'custom'
         ADB_SHELL_STEPS = []
         STEPS_HOST_PRE = []
@@ -404,7 +402,7 @@ class run_custom(AndroidCommand):
             file_url = self.args.command_file
             urlpath = urlparse.urlsplit(file_url).path
             file_name = os.path.basename(urlpath)
-            target_path = os.path.join(config.installdir_android,
+            target_path = os.path.join(self.config.installdir_android,
                                      test_name, file_name)
             STEPS_HOST_PRE = ["wget %s -O %s" % (file_url, file_name)]
             STEPS_ADB_PRE = ["push %s %s" % (file_name, target_path)]
@@ -490,18 +488,13 @@ class run_monkeyrunner(AndroidCommand):
                                  " bundle and finally save the complete bundle"
                                  " to the  specified FILE."))
 
-    def invoke(self):
-        config = get_config()
-        if self.args.serial:
-            serial = self.args.serial
-        else:
-            serial = self.get_device_serial()
-            if not serial:
-                raise LavaCommandError("No device attached")
-        self.adb = ADB(serial)
+    def invoke_sub(self):
+
+        if not utils.check_command_exist('monkeyrunner'):
+            raise LavaCommandError('The command monkeyrunner can not be found')
 
         if self.args.repo_type == 'git':
-            target_dir = mkdtemp(prefix='git_repo', dir=config.tempdir_host)
+            target_dir = mkdtemp(prefix='git_repo', dir=self.config.tempdir_host)
             os.chmod(target_dir, 0755)
             GitRepository(self.args.url).checkout(target_dir)
         else:
@@ -517,7 +510,7 @@ class run_monkeyrunner(AndroidCommand):
 
         tip_msg = ("Run monkeyrunner scripts in following url on device(%s):"
                        "\n\turl=%s") % (
-                       self.args.serial,
+                       self.serial,
                        self.args.url)
 
         self.say_begin(tip_msg)
@@ -530,7 +523,7 @@ class run_monkeyrunner(AndroidCommand):
             if len(test_case_id) > 50:
                 test_case_id = '%s...' % (test_case_id[:50])
             try:
-                sub_bundle = self.run_monkeyrunner_test(script, serial,
+                sub_bundle = self.run_monkeyrunner_test(script, self.serial,
                                                          test_case_id)
                 test_result = {"test_case_id": test_case_id,
                                "result": 'pass'}
@@ -563,7 +556,6 @@ class run_monkeyrunner(AndroidCommand):
         self.say_end(tip_msg)
 
     def run_monkeyrunner_test(self, script, serial, test_case_id=None):
-        config = get_config()
 
         inst = AndroidTestInstaller()
         run = AndroidTestRunner(steps_host_pre=[
@@ -583,11 +575,11 @@ class run_monkeyrunner(AndroidCommand):
         ##The png files here are generated to the host by the monkeyrunner
         ##monkeyrunner is run on host, not on the target
         bundle = {}
-        org_png_file_list = utils.find_files(config.tempdir_host,
+        org_png_file_list = utils.find_files(self.config.tempdir_host,
                                              '.%s' % 'png')
         result_id = test.run(quiet=self.args.quiet)
         if self.args.output:
-            cur_all_png_list = utils.find_files(config.tempdir_host,
+            cur_all_png_list = utils.find_files(self.config.tempdir_host,
                                                 '.%s' % 'png')
             new_png_list = set(cur_all_png_list).difference(org_png_file_list)
             test_id = 'monkeyrunner(%s)' % (test_case_id)
@@ -691,8 +683,8 @@ def generate_bundle(serial=None, result_id=None, test=None,
     if test:
         test_tmp = test
     else:
-        test_tmp = testloader(bundle['test_runs'][0]['test_id'], serial)
-
+        test_tmp = TestProvider().load_test(bundle['test_runs'][0]['test_id'],
+                                             serial)
     if test_id:
         bundle['test_runs'][0]['test_id'] = test_id
 
